@@ -5,15 +5,34 @@ This module provides optimized OLS and variance estimation with an optional
 Rust backend for maximum performance.
 
 The key optimizations are:
-1. scipy.linalg.lstsq with 'gelsy' driver (QR-based, faster than SVD)
+1. scipy.linalg.lstsq with 'gelsd' driver (SVD-based, handles rank-deficient matrices)
 2. Vectorized cluster-robust SE via groupby (eliminates O(n*clusters) loop)
 3. Single interface for all estimators (reduces code duplication)
 4. Optional Rust backend for additional speedup (when available)
+5. R-style rank deficiency handling: detect, warn, and set NA for dropped columns
 
 The Rust backend is automatically used when available, with transparent
 fallback to NumPy/SciPy implementations.
+
+Rank Deficiency Handling
+------------------------
+When a design matrix is rank-deficient (has linearly dependent columns), the OLS
+solution is not unique. This module follows R's `lm()` approach:
+
+1. Detect rank deficiency using pivoted QR decomposition
+2. Identify which columns are linearly dependent
+3. Drop redundant columns from the solve
+4. Set NA (NaN) for coefficients of dropped columns
+5. Warn with clear message listing dropped columns
+6. Compute valid SEs for remaining (identified) coefficients
+
+This is controlled by the `rank_deficient_action` parameter:
+- "warn" (default): Emit warning, set NA for dropped coefficients
+- "error": Raise ValueError with dropped column information
+- "silent": No warning, but still set NA for dropped coefficients
 """
 
+import warnings
 from dataclasses import dataclass
 from typing import Dict, List, Optional, Tuple, Union
 
@@ -21,6 +40,7 @@ import numpy as np
 import pandas as pd
 from scipy import stats
 from scipy.linalg import lstsq as scipy_lstsq
+from scipy.linalg import qr
 
 # Import Rust backend if available (from _backend to avoid circular imports)
 from diff_diff._backend import (
@@ -30,6 +50,276 @@ from diff_diff._backend import (
 )
 
 
+# =============================================================================
+# Utility Functions
+# =============================================================================
+
+
+def _factorize_cluster_ids(cluster_ids: np.ndarray) -> np.ndarray:
+    """
+    Convert cluster IDs to contiguous integer codes for Rust backend.
+
+    Handles string, categorical, or non-contiguous integer cluster IDs by
+    mapping them to contiguous integers starting from 0.
+
+    Parameters
+    ----------
+    cluster_ids : np.ndarray
+        Cluster identifiers (can be strings, integers, or categorical).
+
+    Returns
+    -------
+    np.ndarray
+        Integer cluster codes (dtype int64) suitable for Rust backend.
+    """
+    # Use pandas factorize for efficient conversion of any dtype
+    codes, _ = pd.factorize(cluster_ids)
+    return codes.astype(np.int64)
+
+
+# =============================================================================
+# Rank Deficiency Detection and Handling
+# =============================================================================
+
+
+def _detect_rank_deficiency(
+    X: np.ndarray,
+    rcond: Optional[float] = None,
+) -> Tuple[int, np.ndarray, np.ndarray]:
+    """
+    Detect rank deficiency using pivoted QR decomposition.
+
+    This follows R's lm() approach of using pivoted QR to detect which columns
+    are linearly dependent. The pivoting ensures we drop the "least important"
+    columns (those with smallest contribution to the column space).
+
+    Parameters
+    ----------
+    X : ndarray of shape (n, k)
+        Design matrix.
+    rcond : float, optional
+        Relative condition number threshold for determining rank.
+        Diagonal elements of R smaller than rcond * max(|R_ii|) are treated
+        as zero. If None, uses 1e-07 to match R's qr() default tolerance.
+
+    Returns
+    -------
+    rank : int
+        Numerical rank of the matrix.
+    dropped_cols : ndarray of int
+        Indices of columns that are linearly dependent (should be dropped).
+        Empty if matrix is full rank.
+    pivot : ndarray of int
+        Column permutation from QR decomposition.
+    """
+    n, k = X.shape
+
+    # Compute pivoted QR decomposition: X @ P = Q @ R
+    # P is a permutation matrix, represented as pivot indices
+    Q, R, pivot = qr(X, mode='economic', pivoting=True)
+
+    # Determine rank tolerance
+    # R's qr() uses tol = 1e-07 by default, which is sqrt(eps) ≈ 1.49e-08
+    # We use 1e-07 to match R's lm() behavior for consistency
+    if rcond is None:
+        rcond = 1e-07
+
+    # The diagonal of R contains information about linear independence
+    # After pivoting, |R[i,i]| is decreasing
+    r_diag = np.abs(np.diag(R))
+
+    # Find numerical rank: count singular values above threshold
+    # The threshold is relative to the largest diagonal element
+    if r_diag[0] == 0:
+        rank = 0
+    else:
+        tol = rcond * r_diag[0]
+        rank = int(np.sum(r_diag > tol))
+
+    # Columns after rank position (in pivot order) are linearly dependent
+    # We need to map back to original column indices
+    if rank < k:
+        dropped_cols = np.sort(pivot[rank:])
+    else:
+        dropped_cols = np.array([], dtype=int)
+
+    return rank, dropped_cols, pivot
+
+
+def _format_dropped_columns(
+    dropped_cols: np.ndarray,
+    column_names: Optional[List[str]] = None,
+) -> str:
+    """
+    Format dropped column information for error/warning messages.
+
+    Parameters
+    ----------
+    dropped_cols : ndarray of int
+        Indices of dropped columns.
+    column_names : list of str, optional
+        Names for the columns. If None, uses indices.
+
+    Returns
+    -------
+    str
+        Formatted string describing dropped columns.
+    """
+    if len(dropped_cols) == 0:
+        return ""
+
+    if column_names is not None:
+        names = [column_names[i] if i < len(column_names) else f"column {i}"
+                 for i in dropped_cols]
+        if len(names) == 1:
+            return f"'{names[0]}'"
+        elif len(names) <= 5:
+            return ", ".join(f"'{n}'" for n in names)
+        else:
+            shown = ", ".join(f"'{n}'" for n in names[:5])
+            return f"{shown}, ... and {len(names) - 5} more"
+    else:
+        if len(dropped_cols) == 1:
+            return f"column {dropped_cols[0]}"
+        elif len(dropped_cols) <= 5:
+            return ", ".join(f"column {i}" for i in dropped_cols)
+        else:
+            shown = ", ".join(f"column {i}" for i in dropped_cols[:5])
+            return f"{shown}, ... and {len(dropped_cols) - 5} more"
+
+
+def _expand_coefficients_with_nan(
+    coef_reduced: np.ndarray,
+    k_full: int,
+    kept_cols: np.ndarray,
+) -> np.ndarray:
+    """
+    Expand reduced coefficients to full size, filling dropped columns with NaN.
+
+    Parameters
+    ----------
+    coef_reduced : ndarray of shape (rank,)
+        Coefficients for kept columns only.
+    k_full : int
+        Total number of columns in original design matrix.
+    kept_cols : ndarray of int
+        Indices of columns that were kept.
+
+    Returns
+    -------
+    ndarray of shape (k_full,)
+        Full coefficient vector with NaN for dropped columns.
+    """
+    coef_full = np.full(k_full, np.nan)
+    coef_full[kept_cols] = coef_reduced
+    return coef_full
+
+
+def _expand_vcov_with_nan(
+    vcov_reduced: np.ndarray,
+    k_full: int,
+    kept_cols: np.ndarray,
+) -> np.ndarray:
+    """
+    Expand reduced vcov matrix to full size, filling dropped entries with NaN.
+
+    Parameters
+    ----------
+    vcov_reduced : ndarray of shape (rank, rank)
+        Variance-covariance matrix for kept columns only.
+    k_full : int
+        Total number of columns in original design matrix.
+    kept_cols : ndarray of int
+        Indices of columns that were kept.
+
+    Returns
+    -------
+    ndarray of shape (k_full, k_full)
+        Full vcov matrix with NaN for dropped rows/columns.
+    """
+    vcov_full = np.full((k_full, k_full), np.nan)
+    # Use advanced indexing to fill in the kept entries
+    ix = np.ix_(kept_cols, kept_cols)
+    vcov_full[ix] = vcov_reduced
+    return vcov_full
+
+
+def _solve_ols_rust(
+    X: np.ndarray,
+    y: np.ndarray,
+    *,
+    cluster_ids: Optional[np.ndarray] = None,
+    return_vcov: bool = True,
+    return_fitted: bool = False,
+) -> Union[
+    Tuple[np.ndarray, np.ndarray, Optional[np.ndarray]],
+    Tuple[np.ndarray, np.ndarray, np.ndarray, Optional[np.ndarray]],
+]:
+    """
+    Rust backend implementation of solve_ols for full-rank matrices.
+
+    This is only called when:
+    1. The Rust backend is available
+    2. The design matrix is full rank (no rank deficiency handling needed)
+
+    For rank-deficient matrices, the Python backend is used instead to
+    properly handle R-style NA coefficients for dropped columns.
+
+    Why the backends differ (by design):
+    - Rust uses SVD-based solve (minimum-norm solution for rank-deficient)
+    - Python uses pivoted QR to identify and drop linearly dependent columns
+    - ndarray-linalg doesn't support QR with pivoting, so Rust can't identify
+      which specific columns to drop
+    - For full-rank matrices, both approaches give identical results
+    - For rank-deficient matrices, only Python can provide R-style NA handling
+
+    Parameters
+    ----------
+    X : np.ndarray
+        Design matrix of shape (n, k), must be full rank.
+    y : np.ndarray
+        Response vector of shape (n,).
+    cluster_ids : np.ndarray, optional
+        Cluster identifiers for cluster-robust SEs.
+    return_vcov : bool
+        Whether to compute variance-covariance matrix.
+    return_fitted : bool
+        Whether to return fitted values.
+
+    Returns
+    -------
+    coefficients : np.ndarray
+        OLS coefficients of shape (k,).
+    residuals : np.ndarray
+        Residuals of shape (n,).
+    fitted : np.ndarray, optional
+        Fitted values if return_fitted=True.
+    vcov : np.ndarray, optional
+        Variance-covariance matrix if return_vcov=True.
+    """
+    # Convert cluster_ids to int64 for Rust (handles string/categorical IDs)
+    if cluster_ids is not None:
+        cluster_ids = _factorize_cluster_ids(cluster_ids)
+
+    # Call Rust backend
+    coefficients, residuals, vcov = _rust_solve_ols(
+        X, y, cluster_ids=cluster_ids, return_vcov=return_vcov
+    )
+
+    # Convert to numpy arrays
+    coefficients = np.asarray(coefficients)
+    residuals = np.asarray(residuals)
+    if vcov is not None:
+        vcov = np.asarray(vcov)
+
+    # Return with optional fitted values
+    if return_fitted:
+        fitted = X @ coefficients
+        return coefficients, residuals, fitted, vcov
+    else:
+        return coefficients, residuals, vcov
+
+
 def solve_ols(
     X: np.ndarray,
     y: np.ndarray,
@@ -38,6 +328,9 @@ def solve_ols(
     return_vcov: bool = True,
     return_fitted: bool = False,
     check_finite: bool = True,
+    rank_deficient_action: str = "warn",
+    column_names: Optional[List[str]] = None,
+    skip_rank_check: bool = False,
 ) -> Union[
     Tuple[np.ndarray, np.ndarray, Optional[np.ndarray]],
     Tuple[np.ndarray, np.ndarray, np.ndarray, Optional[np.ndarray]],
@@ -65,24 +358,48 @@ def solve_ols(
     check_finite : bool, default True
         Whether to check that X and y contain only finite values (no NaN/Inf).
         Set to False for faster computation if you are certain your data is clean.
+    rank_deficient_action : str, default "warn"
+        How to handle rank-deficient design matrices:
+        - "warn": Emit warning and set NaN for dropped coefficients (R-style)
+        - "error": Raise ValueError with dropped column information
+        - "silent": No warning, but still set NaN for dropped coefficients
+    column_names : list of str, optional
+        Names for the columns (used in warning/error messages).
+        If None, columns are referred to by their indices.
+    skip_rank_check : bool, default False
+        If True, skip the pivoted QR rank check and use Rust backend directly
+        (when available). This saves O(nk²) computation but will not detect
+        rank-deficient matrices. Use only when you know the design matrix is
+        full rank. If the matrix is actually rank-deficient, results may be
+        incorrect (minimum-norm solution instead of R-style NA handling).
 
     Returns
     -------
     coefficients : ndarray of shape (k,)
-        OLS coefficient estimates.
+        OLS coefficient estimates. For rank-deficient matrices, coefficients
+        of linearly dependent columns are set to NaN.
     residuals : ndarray of shape (n,)
-        Residuals (y - X @ coefficients).
+        Residuals (y - fitted). For rank-deficient matrices, uses only
+        identified coefficients to compute fitted values.
     fitted : ndarray of shape (n,), optional
-        Fitted values (X @ coefficients). Only returned if return_fitted=True.
+        Fitted values. For full-rank matrices, this is X @ coefficients.
+        For rank-deficient matrices, uses only identified coefficients
+        (X_reduced @ coefficients_reduced). Only returned if return_fitted=True.
     vcov : ndarray of shape (k, k) or None
         Variance-covariance matrix (HC1 or cluster-robust).
-        None if return_vcov=False.
+        For rank-deficient matrices, rows/columns for dropped coefficients
+        are filled with NaN. None if return_vcov=False.
 
     Notes
     -----
-    This function uses scipy.linalg.lstsq with the 'gelsy' driver, which is
-    QR-based and typically faster than NumPy's default SVD-based solver for
-    well-conditioned matrices.
+    This function detects rank-deficient matrices using pivoted QR decomposition
+    and handles them following R's lm() approach:
+
+    1. Detect linearly dependent columns via pivoted QR
+    2. Drop redundant columns and solve the reduced system
+    3. Set NaN for coefficients of dropped columns
+    4. Compute valid SEs for identified coefficients only
+    5. Expand vcov matrix with NaN for dropped rows/columns
 
     The cluster-robust standard errors use the sandwich estimator with the
     standard small-sample adjustment: (G/(G-1)) * ((n-1)/(n-k)).
@@ -95,6 +412,15 @@ def solve_ols(
     >>> y = 2 + 3 * X[:, 1] + np.random.randn(100)
     >>> coef, resid, vcov = solve_ols(X, y)
     >>> print(f"Intercept: {coef[0]:.2f}, Slope: {coef[1]:.2f}")
+
+    For rank-deficient matrices with collinear columns:
+
+    >>> X = np.random.randn(100, 3)
+    >>> X[:, 2] = X[:, 0] + X[:, 1]  # Perfect collinearity
+    >>> y = np.random.randn(100)
+    >>> coef, resid, vcov = solve_ols(X, y)  # Emits warning
+    >>> print(np.isnan(coef[2]))  # Dropped column has NaN coefficient
+    True
     """
     # Validate inputs
     X = np.asarray(X, dtype=np.float64)
@@ -117,6 +443,14 @@ def solve_ols(
             "Cannot solve underdetermined system."
         )
 
+    # Validate rank_deficient_action
+    valid_actions = {"warn", "error", "silent"}
+    if rank_deficient_action not in valid_actions:
+        raise ValueError(
+            f"rank_deficient_action must be one of {valid_actions}, "
+            f"got '{rank_deficient_action}'"
+        )
+
     # Check for NaN/Inf values if requested
     if check_finite:
         if not np.isfinite(X).all():
@@ -130,43 +464,87 @@ def solve_ols(
                 "Clean your data or set check_finite=False to skip this check."
             )
 
-    # Use Rust backend if available
-    # Note: Fall back to NumPy if check_finite=False since Rust's LAPACK
-    # doesn't support non-finite values
-    if HAS_RUST_BACKEND and check_finite:
-        # Ensure contiguous arrays for Rust
-        X = np.ascontiguousarray(X, dtype=np.float64)
-        y = np.ascontiguousarray(y, dtype=np.float64)
-
-        # Convert cluster_ids to int64 for Rust (if provided)
-        cluster_ids_int = None
-        if cluster_ids is not None:
-            cluster_ids_int = pd.factorize(cluster_ids)[0].astype(np.int64)
-
-        try:
-            coefficients, residuals, vcov = _rust_solve_ols(
-                X, y, cluster_ids_int, return_vcov
+    # Fast path: skip rank check and use Rust directly when requested
+    # This saves O(nk²) QR overhead but won't detect rank-deficient matrices
+    if skip_rank_check:
+        if HAS_RUST_BACKEND and _rust_solve_ols is not None:
+            return _solve_ols_rust(
+                X, y,
+                cluster_ids=cluster_ids,
+                return_vcov=return_vcov,
+                return_fitted=return_fitted,
             )
-        except ValueError as e:
-            # Translate Rust LAPACK errors to consistent Python error messages
-            error_msg = str(e)
-            if "Matrix inversion failed" in error_msg or "Least squares failed" in error_msg:
-                raise ValueError(
-                    "Design matrix is rank-deficient (singular X'X matrix). "
-                    "This indicates perfect multicollinearity. Check your fixed effects "
-                    "and covariates for linear dependencies."
-                ) from e
-            raise
+        # Fall through to Python without rank check (user guarantees full rank)
+        return _solve_ols_numpy(
+            X, y,
+            cluster_ids=cluster_ids,
+            return_vcov=return_vcov,
+            return_fitted=return_fitted,
+            rank_deficient_action=rank_deficient_action,
+            column_names=column_names,
+            _skip_rank_check=True,
+        )
 
-        if return_fitted:
-            fitted = X @ coefficients
-            return coefficients, residuals, fitted, vcov
+    # Check for rank deficiency using fast pivoted QR decomposition.
+    # This adds O(nk²) overhead but is necessary for:
+    # 1. Detecting which columns to drop (R-style NA handling)
+    # 2. Routing rank-deficient cases to Python (Rust doesn't support pivoted QR)
+    #
+    # Trade-off: ~2x compute cost for full-rank matrices in exchange for proper
+    # rank deficiency handling. For maximum performance on known full-rank data,
+    # set skip_rank_check=True.
+    rank, dropped_cols, pivot = _detect_rank_deficiency(X)
+    is_rank_deficient = len(dropped_cols) > 0
+
+    # Routing strategy:
+    # - Full-rank + Rust available → fast Rust backend (SVD-based solve)
+    # - Rank-deficient → Python backend (proper NA handling, valid SEs)
+    # - No Rust → Python backend (works for all cases)
+    if HAS_RUST_BACKEND and _rust_solve_ols is not None and not is_rank_deficient:
+        result = _solve_ols_rust(
+            X, y,
+            cluster_ids=cluster_ids,
+            return_vcov=return_vcov,
+            return_fitted=return_fitted,
+        )
+
+        # Check for NaN vcov: Rust SVD may detect rank-deficiency that QR missed
+        # for ill-conditioned matrices (QR and SVD have different numerical properties).
+        # When this happens, fall back to Python's R-style handling.
+        vcov = result[-1]  # vcov is always the last element
+        if return_vcov and vcov is not None and np.any(np.isnan(vcov)):
+            warnings.warn(
+                "Rust backend detected ill-conditioned matrix (NaN in variance-covariance). "
+                "Re-running with Python backend for proper rank detection.",
+                UserWarning,
+                stacklevel=2,
+            )
+            # Force fresh rank detection - don't pass cached info since QR
+            # and SVD disagreed about rank. Python's QR will re-detect and
+            # apply R-style NaN handling for dropped columns.
+            return _solve_ols_numpy(
+                X, y,
+                cluster_ids=cluster_ids,
+                return_vcov=return_vcov,
+                return_fitted=return_fitted,
+                rank_deficient_action=rank_deficient_action,
+                column_names=column_names,
+                _precomputed_rank_info=None,  # Force re-detection
+            )
         else:
-            return coefficients, residuals, vcov
+            return result
 
-    # Fallback to NumPy/SciPy implementation
+    # Use NumPy implementation for rank-deficient cases (R-style NA handling)
+    # or when Rust backend is not available
     return _solve_ols_numpy(
-        X, y, cluster_ids=cluster_ids, return_vcov=return_vcov, return_fitted=return_fitted
+        X, y,
+        cluster_ids=cluster_ids,
+        return_vcov=return_vcov,
+        return_fitted=return_fitted,
+        rank_deficient_action=rank_deficient_action,
+        column_names=column_names,
+        # Pass pre-computed rank info to avoid redundant computation
+        _precomputed_rank_info=(rank, dropped_cols, pivot),
     )
 
 
@@ -177,18 +555,20 @@ def _solve_ols_numpy(
     cluster_ids: Optional[np.ndarray] = None,
     return_vcov: bool = True,
     return_fitted: bool = False,
+    rank_deficient_action: str = "warn",
+    column_names: Optional[List[str]] = None,
+    _precomputed_rank_info: Optional[Tuple[int, np.ndarray, np.ndarray]] = None,
+    _skip_rank_check: bool = False,
 ) -> Union[
     Tuple[np.ndarray, np.ndarray, Optional[np.ndarray]],
     Tuple[np.ndarray, np.ndarray, np.ndarray, Optional[np.ndarray]],
 ]:
     """
-    NumPy/SciPy fallback implementation of solve_ols.
+    NumPy/SciPy implementation of solve_ols with R-style rank deficiency handling.
 
-    Uses scipy.linalg.lstsq with 'gelsy' driver (QR with column pivoting)
-    for numerically stable least squares solving. QR decomposition is preferred
-    over normal equations because it doesn't square the condition number of X,
-    making it more robust for ill-conditioned matrices common in DiD designs
-    (e.g., many unit/time fixed effects).
+    Detects rank-deficient matrices using pivoted QR decomposition and handles
+    them following R's lm() approach: drop redundant columns, set NA (NaN) for
+    their coefficients, and compute valid SEs for identified coefficients only.
 
     Parameters
     ----------
@@ -202,32 +582,100 @@ def _solve_ols_numpy(
         Whether to compute variance-covariance matrix.
     return_fitted : bool
         Whether to return fitted values.
+    rank_deficient_action : str
+        How to handle rank deficiency: "warn", "error", or "silent".
+    column_names : list of str, optional
+        Names for the columns (used in warning/error messages).
+    _precomputed_rank_info : tuple, optional
+        Pre-computed (rank, dropped_cols, pivot) from _detect_rank_deficiency.
+        Used internally to avoid redundant computation when called from solve_ols.
+    _skip_rank_check : bool, default False
+        If True, skip rank detection entirely and assume full rank.
+        Used when caller has already determined matrix is full rank.
 
     Returns
     -------
     coefficients : np.ndarray
-        OLS coefficients of shape (k,).
+        OLS coefficients of shape (k,). NaN for dropped columns.
     residuals : np.ndarray
         Residuals of shape (n,).
     fitted : np.ndarray, optional
         Fitted values if return_fitted=True.
     vcov : np.ndarray, optional
-        Variance-covariance matrix if return_vcov=True.
+        Variance-covariance matrix if return_vcov=True. NaN for dropped rows/cols.
     """
-    # Solve OLS using QR decomposition via scipy's optimized LAPACK routines
-    # 'gelsy' uses QR with column pivoting, which is numerically stable even
-    # for ill-conditioned matrices (doesn't square the condition number like
-    # normal equations would)
-    coefficients = scipy_lstsq(X, y, lapack_driver="gelsy", check_finite=False)[0]
+    n, k = X.shape
 
-    # Compute residuals and fitted values
-    fitted = X @ coefficients
-    residuals = y - fitted
+    # Determine rank deficiency status
+    if _skip_rank_check:
+        # Caller guarantees full rank - skip expensive QR decomposition
+        is_rank_deficient = False
+        dropped_cols = np.array([], dtype=int)
+    elif _precomputed_rank_info is not None:
+        # Use pre-computed rank info
+        rank, dropped_cols, pivot = _precomputed_rank_info
+        is_rank_deficient = len(dropped_cols) > 0
+    else:
+        # Compute rank via pivoted QR
+        rank, dropped_cols, pivot = _detect_rank_deficiency(X)
+        is_rank_deficient = len(dropped_cols) > 0
 
-    # Compute variance-covariance matrix if requested
-    vcov = None
-    if return_vcov:
-        vcov = _compute_robust_vcov_numpy(X, residuals, cluster_ids)
+    if is_rank_deficient:
+        # Format dropped column information for messages
+        dropped_str = _format_dropped_columns(dropped_cols, column_names)
+
+        if rank_deficient_action == "error":
+            raise ValueError(
+                f"Design matrix is rank-deficient. {k - rank} of {k} columns are "
+                f"linearly dependent and cannot be uniquely estimated: {dropped_str}. "
+                "This indicates multicollinearity in your model specification."
+            )
+        elif rank_deficient_action == "warn":
+            warnings.warn(
+                f"Rank-deficient design matrix: dropping {k - rank} of {k} columns "
+                f"({dropped_str}). Coefficients for these columns are set to NA. "
+                "This may indicate multicollinearity in your model specification.",
+                UserWarning,
+                stacklevel=3,  # Point to user code that called solve_ols
+            )
+        # else: "silent" - no warning
+
+        # Extract kept columns for the reduced solve
+        kept_cols = np.array([i for i in range(k) if i not in dropped_cols])
+        X_reduced = X[:, kept_cols]
+
+        # Solve the reduced system (now full-rank)
+        # Use cond=1e-07 for consistency with Rust backend and QR rank tolerance
+        coefficients_reduced = scipy_lstsq(
+            X_reduced, y, lapack_driver="gelsd", check_finite=False, cond=1e-07
+        )[0]
+
+        # Expand coefficients to full size with NaN for dropped columns
+        coefficients = _expand_coefficients_with_nan(coefficients_reduced, k, kept_cols)
+
+        # Compute residuals using only the identified coefficients
+        # Note: Dropped coefficients are NaN, so we use the reduced form
+        fitted = X_reduced @ coefficients_reduced
+        residuals = y - fitted
+
+        # Compute variance-covariance matrix for reduced system, then expand
+        vcov = None
+        if return_vcov:
+            vcov_reduced = _compute_robust_vcov_numpy(X_reduced, residuals, cluster_ids)
+            vcov = _expand_vcov_with_nan(vcov_reduced, k, kept_cols)
+    else:
+        # Full-rank case: proceed normally
+        # Use cond=1e-07 for consistency with Rust backend and QR rank tolerance
+        coefficients = scipy_lstsq(X, y, lapack_driver="gelsd", check_finite=False, cond=1e-07)[0]
+
+        # Compute residuals and fitted values
+        fitted = X @ coefficients
+        residuals = y - fitted
+
+        # Compute variance-covariance matrix if requested
+        vcov = None
+        if return_vcov:
+            vcov = _compute_robust_vcov_numpy(X, residuals, cluster_ids)
 
     if return_fitted:
         return coefficients, residuals, fitted, vcov
@@ -475,12 +923,22 @@ class InferenceResult:
     alpha: float = 0.05
 
     def is_significant(self, alpha: Optional[float] = None) -> bool:
-        """Check if the coefficient is statistically significant."""
+        """Check if the coefficient is statistically significant.
+
+        Returns False for NaN p-values (unidentified coefficients).
+        """
+        if np.isnan(self.p_value):
+            return False
         threshold = alpha if alpha is not None else self.alpha
         return self.p_value < threshold
 
     def significance_stars(self) -> str:
-        """Return significance stars based on p-value."""
+        """Return significance stars based on p-value.
+
+        Returns empty string for NaN p-values (unidentified coefficients).
+        """
+        if np.isnan(self.p_value):
+            return ""
         if self.p_value < 0.001:
             return "***"
         elif self.p_value < 0.01:
@@ -526,6 +984,11 @@ class LinearRegression:
         Overrides the `robust` parameter if provided.
     alpha : float, default 0.05
         Significance level for confidence intervals.
+    rank_deficient_action : str, default "warn"
+        Action when design matrix is rank-deficient (linearly dependent columns):
+        - "warn": Issue warning and drop linearly dependent columns (default)
+        - "error": Raise ValueError
+        - "silent": Drop columns silently without warning
 
     Attributes
     ----------
@@ -541,8 +1004,11 @@ class LinearRegression:
         Number of observations (available after fit).
     n_params_ : int
         Number of parameters including intercept (available after fit).
+    n_params_effective_ : int
+        Effective number of parameters after dropping linearly dependent columns.
+        Equals n_params_ for full-rank matrices (available after fit).
     df_ : int
-        Degrees of freedom (n - k) (available after fit).
+        Degrees of freedom (n - n_params_effective) (available after fit).
 
     Examples
     --------
@@ -577,11 +1043,13 @@ class LinearRegression:
         robust: bool = True,
         cluster_ids: Optional[np.ndarray] = None,
         alpha: float = 0.05,
+        rank_deficient_action: str = "warn",
     ):
         self.include_intercept = include_intercept
         self.robust = robust
         self.cluster_ids = cluster_ids
         self.alpha = alpha
+        self.rank_deficient_action = rank_deficient_action
 
         # Fitted attributes (set by fit())
         self.coefficients_: Optional[np.ndarray] = None
@@ -592,6 +1060,7 @@ class LinearRegression:
         self._X: Optional[np.ndarray] = None
         self.n_obs_: Optional[int] = None
         self.n_params_: Optional[int] = None
+        self.n_params_effective_: Optional[int] = None
         self.df_: Optional[int] = None
 
     def fit(
@@ -643,6 +1112,7 @@ class LinearRegression:
                 cluster_ids=effective_cluster_ids,
                 return_fitted=True,
                 return_vcov=compute_vcov,
+                rank_deficient_action=self.rank_deficient_action,
             )
         else:
             # Classical OLS - compute vcov separately
@@ -650,15 +1120,37 @@ class LinearRegression:
                 X, y,
                 return_fitted=True,
                 return_vcov=False,
+                rank_deficient_action=self.rank_deficient_action,
             )
             # Compute classical OLS variance-covariance matrix
+            # Handle rank-deficient case: use effective rank for df
             n, k = X.shape
-            mse = np.sum(residuals**2) / (n - k)
-            try:
-                vcov = np.linalg.solve(X.T @ X, mse * np.eye(k))
-            except np.linalg.LinAlgError:
-                # Fall back to pseudo-inverse for singular matrices
-                vcov = np.linalg.pinv(X.T @ X) * mse
+            nan_mask = np.isnan(coefficients)
+            k_effective = k - np.sum(nan_mask)  # Number of identified coefficients
+
+            if k_effective == 0:
+                # All coefficients dropped - no valid inference
+                vcov = np.full((k, k), np.nan)
+            elif np.any(nan_mask):
+                # Rank-deficient: compute vcov for identified coefficients only
+                kept_cols = np.where(~nan_mask)[0]
+                X_reduced = X[:, kept_cols]
+                mse = np.sum(residuals**2) / (n - k_effective)
+                try:
+                    vcov_reduced = np.linalg.solve(
+                        X_reduced.T @ X_reduced, mse * np.eye(k_effective)
+                    )
+                except np.linalg.LinAlgError:
+                    vcov_reduced = np.linalg.pinv(X_reduced.T @ X_reduced) * mse
+                # Expand to full size with NaN for dropped columns
+                vcov = _expand_vcov_with_nan(vcov_reduced, k, kept_cols)
+            else:
+                # Full rank: standard computation
+                mse = np.sum(residuals**2) / (n - k)
+                try:
+                    vcov = np.linalg.solve(X.T @ X, mse * np.eye(k))
+                except np.linalg.LinAlgError:
+                    vcov = np.linalg.pinv(X.T @ X) * mse
 
         # Store fitted attributes
         self.coefficients_ = coefficients
@@ -669,7 +1161,12 @@ class LinearRegression:
         self._X = X
         self.n_obs_ = X.shape[0]
         self.n_params_ = X.shape[1]
-        self.df_ = self.n_obs_ - self.n_params_ - df_adjustment
+
+        # Compute effective number of parameters (excluding dropped columns)
+        # This is needed for correct degrees of freedom in inference
+        nan_mask = np.isnan(coefficients)
+        self.n_params_effective_ = int(self.n_params_ - np.sum(nan_mask))
+        self.df_ = self.n_obs_ - self.n_params_effective_ - df_adjustment
 
         return self
 
@@ -876,10 +1373,18 @@ class LinearRegression:
         -------
         float
             R-squared value.
+
+        Notes
+        -----
+        For rank-deficient fits, adjusted R² uses the effective number of
+        parameters (excluding dropped columns) for consistency with the
+        corrected degrees of freedom.
         """
         self._check_fitted()
+        # Use effective params for adjusted R² to match df correction
+        n_params = self.n_params_effective_ if adjusted else self.n_params_
         return compute_r_squared(
-            self._y, self.residuals_, adjusted=adjusted, n_params=self.n_params_
+            self._y, self.residuals_, adjusted=adjusted, n_params=n_params
         )
 
     def predict(self, X: np.ndarray) -> np.ndarray:
@@ -896,6 +1401,12 @@ class LinearRegression:
         -------
         ndarray
             Predicted values.
+
+        Notes
+        -----
+        For rank-deficient fits where some coefficients are NaN, predictions
+        use only the identified (non-NaN) coefficients. This is equivalent to
+        treating dropped columns as having zero coefficients.
         """
         self._check_fitted()
         X = np.asarray(X, dtype=np.float64)
@@ -903,7 +1414,12 @@ class LinearRegression:
         if self.include_intercept:
             X = np.column_stack([np.ones(X.shape[0]), X])
 
-        return X @ self.coefficients_
+        # Handle rank-deficient case: use only identified coefficients
+        # Replace NaN with 0 so they don't contribute to prediction
+        coef = self.coefficients_.copy()
+        coef[np.isnan(coef)] = 0.0
+
+        return X @ coef
 
 
 # =============================================================================
