@@ -248,6 +248,7 @@ class CallawaySantAnnaBootstrapMixin:
     bootstrap_weight_type: str
     alpha: float
     seed: Optional[int]
+    anticipation: int
 
     def _run_multiplier_bootstrap(
         self,
@@ -310,15 +311,40 @@ class CallawaySantAnnaBootstrapMixin:
         gt_pairs = list(group_time_effects.keys())
         n_gt = len(gt_pairs)
 
-        # Compute aggregation weights for overall ATT
-        overall_weights = np.array([
+        # Identify post-treatment (g,t) pairs for overall ATT
+        # Pre-treatment effects are for parallel trends assessment, not aggregated
+        post_treatment_mask = np.array([
+            t >= g - self.anticipation for (g, t) in gt_pairs
+        ])
+        post_treatment_indices = np.where(post_treatment_mask)[0]
+
+        # Compute aggregation weights for overall ATT (post-treatment only)
+        all_n_treated = np.array([
             group_time_effects[gt]['n_treated'] for gt in gt_pairs
         ], dtype=float)
-        overall_weights = overall_weights / np.sum(overall_weights)
+        post_n_treated = all_n_treated[post_treatment_mask]
+
+        # Flag to skip overall ATT aggregation when no post-treatment effects
+        # But continue bootstrap for per-effect SEs (pre-treatment effects need bootstrap SEs too)
+        skip_overall_aggregation = False
+        if len(post_treatment_indices) == 0:
+            warnings.warn(
+                "No post-treatment effects for bootstrap aggregation. "
+                "Overall ATT statistics will be NaN, but per-effect SEs will be computed.",
+                UserWarning,
+                stacklevel=2
+            )
+            skip_overall_aggregation = True
+            overall_weights_post = np.array([])
+        else:
+            overall_weights_post = post_n_treated / np.sum(post_n_treated)
 
         # Original point estimates
         original_atts = np.array([group_time_effects[gt]['effect'] for gt in gt_pairs])
-        original_overall = np.sum(overall_weights * original_atts)
+        if skip_overall_aggregation:
+            original_overall = np.nan
+        else:
+            original_overall = np.sum(overall_weights_post * original_atts[post_treatment_mask])
 
         # Prepare event study and group aggregation info if needed
         event_study_info = None
@@ -372,18 +398,27 @@ class CallawaySantAnnaBootstrapMixin:
 
             # Vectorized perturbation: matrix-vector multiply
             # Shape: (n_bootstrap,)
-            perturbations = (
-                treated_weights @ treated_inf +
-                control_weights @ control_inf
-            )
+            # Suppress RuntimeWarnings for edge cases (small samples, extreme weights)
+            with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
+                perturbations = (
+                    treated_weights @ treated_inf +
+                    control_weights @ control_inf
+                )
 
+            # Let non-finite values propagate - they will be handled at statistics computation
             bootstrap_atts_gt[:, j] = original_atts[j] + perturbations
 
-        # Vectorized overall ATT: matrix-vector multiply
+        # Vectorized overall ATT: matrix-vector multiply (post-treatment only)
         # Shape: (n_bootstrap,)
-        bootstrap_overall = bootstrap_atts_gt @ overall_weights
+        if skip_overall_aggregation:
+            bootstrap_overall = np.full(self.n_bootstrap, np.nan)
+        else:
+            # Suppress RuntimeWarnings for edge cases - non-finite values handled at statistics computation
+            with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
+                bootstrap_overall = bootstrap_atts_gt[:, post_treatment_indices] @ overall_weights_post
 
         # Vectorized event study aggregation
+        # Non-finite values handled at statistics computation stage
         rel_periods: List[int] = []
         bootstrap_event_study: Optional[Dict[int, np.ndarray]] = None
         if event_study_info is not None:
@@ -394,9 +429,12 @@ class CallawaySantAnnaBootstrapMixin:
                 gt_indices = agg_info['gt_indices']
                 weights = agg_info['weights']
                 # Vectorized: select columns and multiply by weights
-                bootstrap_event_study[e] = bootstrap_atts_gt[:, gt_indices] @ weights
+                # Suppress RuntimeWarnings for edge cases
+                with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
+                    bootstrap_event_study[e] = bootstrap_atts_gt[:, gt_indices] @ weights
 
         # Vectorized group aggregation
+        # Non-finite values handled at statistics computation stage
         group_list: List[Any] = []
         bootstrap_group: Optional[Dict[Any, np.ndarray]] = None
         if group_agg_info is not None:
@@ -406,7 +444,9 @@ class CallawaySantAnnaBootstrapMixin:
                 agg_info = group_agg_info[g]
                 gt_indices = agg_info['gt_indices']
                 weights = agg_info['weights']
-                bootstrap_group[g] = bootstrap_atts_gt[:, gt_indices] @ weights
+                # Suppress RuntimeWarnings for edge cases
+                with np.errstate(divide='ignore', invalid='ignore', over='ignore'):
+                    bootstrap_group[g] = bootstrap_atts_gt[:, gt_indices] @ weights
 
         # Compute bootstrap statistics for ATT(g,t)
         gt_ses = {}
@@ -415,16 +455,23 @@ class CallawaySantAnnaBootstrapMixin:
 
         for j, gt in enumerate(gt_pairs):
             se, ci, p_value = self._compute_effect_bootstrap_stats(
-                original_atts[j], bootstrap_atts_gt[:, j]
+                original_atts[j], bootstrap_atts_gt[:, j],
+                context=f"ATT(g={gt[0]}, t={gt[1]})"
             )
             gt_ses[gt] = se
             gt_cis[gt] = ci
             gt_p_values[gt] = p_value
 
         # Compute bootstrap statistics for overall ATT
-        overall_se, overall_ci, overall_p_value = self._compute_effect_bootstrap_stats(
-            original_overall, bootstrap_overall
-        )
+        if skip_overall_aggregation:
+            overall_se = np.nan
+            overall_ci = (np.nan, np.nan)
+            overall_p_value = np.nan
+        else:
+            overall_se, overall_ci, overall_p_value = self._compute_effect_bootstrap_stats(
+                original_overall, bootstrap_overall,
+                context="overall ATT"
+            )
 
         # Compute bootstrap statistics for event study effects
         event_study_ses = None
@@ -438,7 +485,8 @@ class CallawaySantAnnaBootstrapMixin:
 
             for e in rel_periods:
                 se, ci, p_value = self._compute_effect_bootstrap_stats(
-                    event_study_info[e]['effect'], bootstrap_event_study[e]
+                    event_study_info[e]['effect'], bootstrap_event_study[e],
+                    context=f"event study (e={e})"
                 )
                 event_study_ses[e] = se
                 event_study_cis[e] = ci
@@ -456,7 +504,8 @@ class CallawaySantAnnaBootstrapMixin:
 
             for g in group_list:
                 se, ci, p_value = self._compute_effect_bootstrap_stats(
-                    group_agg_info[g]['effect'], bootstrap_group[g]
+                    group_agg_info[g]['effect'], bootstrap_group[g],
+                    context=f"group effect (g={g})"
                 )
                 group_effect_ses[g] = se
                 group_effect_cis[g] = ci
@@ -549,10 +598,10 @@ class CallawaySantAnnaBootstrapMixin:
         result = {}
 
         for g in treatment_groups:
-            # Get all effects for this group (post-treatment only: t >= g)
+            # Get all effects for this group (post-treatment only: t >= g - anticipation)
             group_data = []
             for j, (gg, t) in enumerate(gt_pairs):
-                if gg == g and t >= g:
+                if gg == g and t >= g - self.anticipation:
                     group_data.append((
                         j,
                         group_time_effects[(gg, t)]['effect'],
@@ -590,6 +639,7 @@ class CallawaySantAnnaBootstrapMixin:
         self,
         original_effect: float,
         boot_dist: np.ndarray,
+        n_valid: Optional[int] = None,
     ) -> float:
         """
         Compute two-sided bootstrap p-value.
@@ -597,6 +647,22 @@ class CallawaySantAnnaBootstrapMixin:
         Uses the percentile method: p-value is the proportion of bootstrap
         estimates on the opposite side of zero from the original estimate,
         doubled for two-sided test.
+
+        Parameters
+        ----------
+        original_effect : float
+            Original point estimate.
+        boot_dist : np.ndarray
+            Bootstrap distribution of the effect.
+        n_valid : int, optional
+            Number of valid bootstrap samples. If None, uses self.n_bootstrap.
+            Use this when boot_dist has already been filtered for non-finite values
+            to ensure the p-value floor is based on the actual valid sample count.
+
+        Returns
+        -------
+        float
+            Two-sided bootstrap p-value.
         """
         if original_effect >= 0:
             # Proportion of bootstrap estimates <= 0
@@ -608,8 +674,9 @@ class CallawaySantAnnaBootstrapMixin:
         # Two-sided p-value
         p_value = min(2 * p_one_sided, 1.0)
 
-        # Ensure minimum p-value
-        p_value = max(p_value, 1 / (self.n_bootstrap + 1))
+        # Ensure minimum p-value using n_valid if provided, otherwise n_bootstrap
+        n_for_floor = n_valid if n_valid is not None else self.n_bootstrap
+        p_value = max(p_value, 1 / (n_for_floor + 1))
 
         return float(p_value)
 
@@ -617,16 +684,23 @@ class CallawaySantAnnaBootstrapMixin:
         self,
         original_effect: float,
         boot_dist: np.ndarray,
+        context: str = "bootstrap distribution",
     ) -> Tuple[float, Tuple[float, float], float]:
         """
         Compute bootstrap statistics for a single effect.
 
+        Non-finite bootstrap samples are dropped and a warning is issued if any
+        are present. If too few valid samples remain (<50%), returns NaN for all
+        statistics to signal invalid inference.
+
         Parameters
         ----------
         original_effect : float
             Original point estimate.
         boot_dist : np.ndarray
             Bootstrap distribution of the effect.
+        context : str, optional
+            Description for warning messages, by default "bootstrap distribution".
 
         Returns
         -------
@@ -637,7 +711,41 @@ class CallawaySantAnnaBootstrapMixin:
         p_value : float
             Bootstrap p-value.
         """
-        se = float(np.std(boot_dist, ddof=1))
-        ci = self._compute_percentile_ci(boot_dist, self.alpha)
-        p_value = self._compute_bootstrap_pvalue(original_effect, boot_dist)
+        # Filter out non-finite values
+        finite_mask = np.isfinite(boot_dist)
+        n_valid = np.sum(finite_mask)
+        n_total = len(boot_dist)
+
+        if n_valid < n_total:
+            import warnings
+            n_nonfinite = n_total - n_valid
+            warnings.warn(
+                f"Dropping {n_nonfinite}/{n_total} non-finite bootstrap samples in {context}. "
+                "This may occur with very small samples or extreme weights. "
+                "Bootstrap estimates based on remaining valid samples.",
+                RuntimeWarning,
+                stacklevel=3
+            )
+
+        # Check if we have enough valid samples
+        if n_valid < n_total * 0.5:
+            import warnings
+            warnings.warn(
+                f"Too few valid bootstrap samples ({n_valid}/{n_total}) in {context}. "
+                "Returning NaN for SE/CI/p-value to signal invalid inference.",
+                RuntimeWarning,
+                stacklevel=3
+            )
+            return np.nan, (np.nan, np.nan), np.nan
+
+        # Use only valid samples
+        valid_dist = boot_dist[finite_mask]
+        n_valid_bootstrap = len(valid_dist)
+
+        se = float(np.std(valid_dist, ddof=1))
+        ci = self._compute_percentile_ci(valid_dist, self.alpha)
+
+        # Compute p-value using shared method with correct floor based on valid sample count
+        p_value = self._compute_bootstrap_pvalue(original_effect, valid_dist, n_valid=n_valid_bootstrap)
+
         return se, ci, p_value
